#include <allegro.h>
#include <alleggl.h>

#include "input_map.h"
#include "util.h"
#include "parody.h"
#include "modes.h"
#include "player.h"
#include "stage.h"
#include "troll.h"
#include "carriers.h"
#include "egg.h"
#include "message.h"




#define MAX_MESSAGE_TIME 80
#define BATTLE_FONT_SIZE 90



typedef struct BATTLE BATTLE;



typedef enum {
    STARTING,
    PLAYING,
    WINNING,
    LOSING,
    PAUSED,
    OPENING_W,
    OPENING_L,
    CLOSING,
} BATTLE_STATUS;



struct BATTLE {
    BATTLE_STATUS status;
    int timer;
};



static BATTLE battle;
static FONT *battle_font = NULL;


int init_battle(DATAFILE *data) {

    if (!init_stage(data)) return 0;
    if (!init_troll(data)) return 0;
    if (!init_carriers(data)) return 0;
    if (!init_player(data)) return 0;
    if (!init_egg(data)) return 0;
    if (!init_messages(data)) return 0;

    battle.status = STARTING;
    battle.timer = MAX_MESSAGE_TIME;

    allegro_gl_set_texture_format(GL_ALPHA8);
    battle_font = allegro_gl_convert_allegro_font((FONT *) data[DAT_FONT_SMALL].dat,
	                                              AGL_FONT_TYPE_TEXTURED, BATTLE_FONT_SIZE);

    if (!battle_font) {
        return 0;
    }

    return 1;
}



void set_battle_level(int level) {
    troll_set_difficulty(level);
}



RESULT update_battle() {
    INPUT_MAP *map = get_input_map();

    if (battle.status == PLAYING) {
        update_stage();
        update_troll();
        update_carriers();
        update_player();
        update_egg();
        update_message(); 

        if (troll_is_defeated()) {
            battle.timer = MAX_MESSAGE_TIME;
            battle.status = OPENING_W;
        }
        if (troll_get_distance() <= 0.0 || player_fell()) {
            battle.timer = MAX_MESSAGE_TIME;
            battle.status = OPENING_L;
        }
    }

    if (battle.timer > 0) {
        --battle.timer;
    } else {
        switch (battle.status) {
            case STARTING: battle.status = CLOSING; break;
            case OPENING_W: battle.status = WINNING; break;
            case OPENING_L: battle.status = LOSING; break;
            case CLOSING: battle.status = PLAYING; break;
        }
        battle.timer = MAX_MESSAGE_TIME;
    }

    if (battle.status == WINNING || battle.status == LOSING) {
        if (key[map->shoot] || key[map->jump]) {
            return battle.status == WINNING ? RES_WIN : RES_LOSE;
        }
    }

    return RES_CONTINUE;
}



void draw_battle() {

    char *line1, *line2;
    float x1, x2, r;

    draw_stage_bg();
    draw_troll();
    draw_egg();
    draw_stage();
    draw_carriers();
    if (battle.status == PLAYING) {
        draw_player();
    }
    draw_player_stats();
    draw_troll_advance();
    draw_message();

    switch (battle.status) {
        case STARTING:
            x1 = -0.75;
            x2 = -0.7;
            line1 = "HIT THE TROLL";
            line2 = "ON THE NOSE";
            break;
        case LOSING:
            x1 = -0.5;
            line1 = "YOU LOSE";
            line2 = NULL;
            break;
        case WINNING:
            x1 = -0.5;
            line1 = "YOU WIN!";
            line2 = NULL;
            break;
    }
    if (battle.status == STARTING || battle.status == LOSING || battle.status == WINNING) {
        glClear(GL_COLOR_BUFFER_BIT);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glEnable(GL_BLEND);
        allegro_gl_printf(battle_font, x1,  0.1, 0, makecol(255,255,255), line1);
        allegro_gl_printf(battle_font, x2, -0.1, 0, makecol(255,255,255), line2);
        glDisable(GL_BLEND);
        glColor4f(1.0, 1.0, 1.0, 1.0);
    }
    if (battle.status == CLOSING || battle.status == OPENING_L || battle.status == OPENING_W) {
        glBegin(GL_POLYGON);
        glColor4f(0.0, 0.0, 0.0, 1.0);
        r = (battle.status == CLOSING) ? 0.0 : -1;
        r += ((float) battle.timer / MAX_MESSAGE_TIME);
        my_draw_circle(0.0, 0.0, r);
        glColor4f(1.0, 1.0, 1.0, 1.0);
        glEnd();
    }
}



void end_battle() {
    end_stage();
    end_troll();
    end_carriers();
    end_player();
    end_egg();
    end_messages();
    if (battle_font) {
        allegro_gl_destroy_font(battle_font);
    }
}
