#include <allegro.h>
#include <alleggl.h>
#include <stdlib.h>

#include "parody.h"
#include "resources.h"
#include "util.h"
#include "collision.h"
#include "carriers.h"

#define CARRIER_BASE_LINE 0.5
#define MAX_OFF_TIME 100
#define MAX_POP_TIME 40



typedef struct CARRIER CARRIER;



struct CARRIER {
    float x, y;
    float r;
    int active;
    int dir;
    float phase;
    float frec;
    GLuint img;
    int off_timer;
    int popping;
    float pop_radio;
    float alpha;
    int pop_timer;
};



static CARRIER carrier;



int init_carriers(DATAFILE *data) {
    BITMAP *bmp;

    carrier.x = 0;
    carrier.y = 0;
    carrier.r = 0.1;
    carrier.active = 0;
    carrier.dir = -1;
    carrier.phase = 0.0;
    carrier.frec = 0.0;
    carrier.off_timer = MAX_OFF_TIME;
    carrier.popping = 0;
    carrier.pop_radio = 0.0;
    carrier.pop_timer = MAX_POP_TIME;
    carrier.alpha = 1.0;

    carrier.img = get_resource(DAT_BALLOON);


    return 1;
}



COLLISION_RESULT carriers_get_collision(COLLISION_DATA *c) {
    COLLISION_DATA carrier_c;
    COLLISION_RESULT res;
    carrier_c.x = carrier.x;
    carrier_c.y = carrier.y;
    carrier_c.r = carrier.r;

    res = check_collision(c, &carrier_c);

    res.collision = res.collision && carrier.active;

    if (res.collision) {
        carrier.active = 0;
        carrier.popping = 1;
        carrier.pop_radio = 0.0;
    }

    return res;
}



void update_carriers() {
    if (carrier.popping) {
        if (carrier.pop_timer > 0) {
            --carrier.pop_timer;
            carrier.pop_radio += 0.01;
            carrier.alpha -= 1.0 / MAX_POP_TIME;
        } else {
            carrier.popping = 0;
            carrier.pop_timer = MAX_POP_TIME;
            carrier.alpha = 1.0;
        }
    }

    if (!carrier.active) {
        if (carrier.off_timer > 0) {
            --carrier.off_timer;
            return;
        } else {
            carrier.off_timer = MAX_OFF_TIME;
        }

        if (carrier.dir < 0) {
            carrier.dir = 1;
        } else {
            carrier.dir = -1;
        }
        carrier.x = carrier.dir - (0.01 * carrier.dir);
        carrier.y = CARRIER_BASE_LINE;
        carrier.active = 1;
        carrier.phase = (rand() % 2) ? 1 : -1;
        carrier.frec = rand() % 5 + 1;
    } else {
        carrier.x += 0.01 * -carrier.dir;
        carrier.y = CARRIER_BASE_LINE + 0.2 * sin(carrier.phase + carrier.x * M_PI * carrier.frec);
        if (carrier.x > 1 || carrier.x < -1) {
            carrier.active = 0;
        }
    }
}



void draw_carriers() {

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	// Additive blending


    if (carrier.popping) {
        glBegin(GL_POLYGON);
            glColor4f(0.5, 0.0, 0.5, carrier.alpha);
            my_draw_circle(carrier.x, carrier.y, carrier.pop_radio);
        glEnd();
    }

    if (carrier.active || carrier.popping) {
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_ALPHA_TEST);
        glBindTexture(GL_TEXTURE_2D, carrier.img);
        glBegin(GL_QUADS);
            glColor4f(1.0, 1.0, 1.0, carrier.alpha);
            glTexCoord2i(0, 1);		//Define texture coordinates
            glVertex2f(carrier.x - carrier.r, carrier.y + carrier.r);
            glTexCoord2i(0, 0);
            glVertex2f(carrier.x - carrier.r, carrier.y - carrier.r);
            glTexCoord2i(1, 0);
            glVertex2f(carrier.x + carrier.r, carrier.y - carrier.r);
            glTexCoord2i(1, 1);
            glVertex2f(carrier.x + carrier.r, carrier.y + carrier.r);
        glEnd();
    	glBindTexture(GL_TEXTURE_2D, 0);
    	glDisable(GL_ALPHA_TEST);

        /* glBegin(GL_LINE_LOOP);
        my_draw_circle(carrier.x, carrier.y, carrier.r);
        glEnd(); */
    }

    glDisable(GL_BLEND);

    glColor4f(1.0, 1.0, 1.0, 1.0);
}



void end_carriers() {
}

