#include <allegro.h>
#include <alleggl.h>
#include <math.h>

#include "parody.h"
#include "resources.h"
#include "util.h"
#include "egg.h"
#include "troll.h"
#include "message.h"

#define MAX_EGGS 3
#define EGG_LIST_R 0.1
#define EGG_LIST_H EGG_LIST_R
#define EGG_LIST_W ((EGG_LIST_R) * (MAX_EGGS + 1))
#define EGG_LIST_X 0.90
#define EGG_LIST_Y 0.95
#define MAX_EXPLOSION_TIME 40



typedef struct EGG EGG;



struct EGG {
    int count;
    float x, y;
    float r;
    float alpha;
    float distance;
    float mid_dist;
    float explosion_radio;
    int shooting;
    int exploding;
    int explosion_timer;
    GLuint img;
};



static EGG egg;



int init_egg(DATAFILE *data) {
    BITMAP *bmp;

    egg.count = 0;
    egg.x = 0.0;
    egg.y = 0.0;
    egg.r = 0.1;
    egg.alpha = 1.0;
    egg.distance = 0.0;
    egg.mid_dist = 0.0;
    egg.explosion_radio = 0.0;
    egg.shooting = 0;
    egg.exploding = 0;
    egg.explosion_timer = MAX_EXPLOSION_TIME;

    egg.img = get_resource(DAT_EGG);


    return 1;
}



void add_egg() {
    if (egg.count < MAX_EGGS) {
        egg.count++;
    }
}


int egg_shoot(float x, float y) {
    if (egg.count > 0) {
        egg.count--;
        egg.shooting = 1;
        egg.x = x;
        egg.y = y;
        egg.distance = 0.0;
        egg.mid_dist = troll_get_distance() / 2.0;
        return 1;
    }
    return 0;
}



int egg_is_shooting() {
    return egg.shooting || egg.exploding;
}



void update_egg() {

    float vy;
    COLLISION_DATA c;
    COLLISION_RESULT res;

    if (egg.exploding) {
        if (egg.explosion_timer > 0) {
            --egg.explosion_timer;
            egg.explosion_radio += 0.01;
            egg.alpha -= 1.0 / MAX_EXPLOSION_TIME;
        } else {
            egg.exploding = 0;
            egg.explosion_radio = 0.0;
            egg.explosion_timer = MAX_EXPLOSION_TIME;
            egg.alpha = 1.0;
        }
    }

    if (egg.shooting) {

        // ugly hack
        egg.r = 0.05 + 0.05 * (TROLL_TOTAL_DISTANCE - egg.distance);

        vy = egg.mid_dist - egg.distance;

        egg.distance += 0.01;

        if (fabs(vy) > 0.01) {
            vy = (vy < 0.0 ? -0.005 : 0.01);
        }

        egg.y += vy;

        if (egg.distance > troll_get_distance()) {

            c.x = egg.x;
            c.y = egg.y;
            c.r = egg.r;
            res = troll_get_collision(&c);

            if (res.collision) {
                egg.exploding = 1;
                set_message(egg.x, egg.y - egg.r, "HIT!");
            } else {
                set_message(egg.x, egg.y - egg.r, "MISS!");
            }

            egg.shooting = 0;
        }

    }
}



void draw_egg() {

    int i;
    float x;

    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	// Additive blending

    glBegin(GL_POLYGON);
        glColor4f(0.8, 0.2, 0.2, 0.5);
        glVertex2f(EGG_LIST_X, EGG_LIST_Y);
        glVertex2f(EGG_LIST_X, EGG_LIST_Y - EGG_LIST_H);
        glVertex2f(EGG_LIST_X - EGG_LIST_W, EGG_LIST_Y - EGG_LIST_H);
        glVertex2f(EGG_LIST_X - EGG_LIST_W, EGG_LIST_Y);
        glColor4f(1.0, 1.0, 1.0, 1.0);
    glEnd();

    for (i = 0; i < egg.count; i++) {
        x = EGG_LIST_X - EGG_LIST_R * 1.5 * i;
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_ALPHA_TEST);
        glBindTexture(GL_TEXTURE_2D, egg.img);
        glBegin(GL_QUADS);
            glColor4f(1.0, 1.0, 1.0, 1.0);
            glTexCoord2i(0, 1);		//Define texture coordinates
            glVertex2f(x - EGG_LIST_R, EGG_LIST_Y + 0 * EGG_LIST_R);
            glTexCoord2i(0, 0);
            glVertex2f(x - EGG_LIST_R, EGG_LIST_Y - 2 * EGG_LIST_R);
            glTexCoord2i(1, 0);
            glVertex2f(x + EGG_LIST_R, EGG_LIST_Y - 2 * EGG_LIST_R);
            glTexCoord2i(1, 1);
            glVertex2f(x + EGG_LIST_R, EGG_LIST_Y + 0 * EGG_LIST_R);
        glEnd();
    	glBindTexture(GL_TEXTURE_2D, 0);
    	glDisable(GL_ALPHA_TEST);
    }

    if (egg.exploding) {
        glBegin(GL_POLYGON);
            glColor4f(1.0, 0.0, 0.0, egg.alpha);
            my_draw_circle(egg.x, egg.y, egg.explosion_radio);
        glEnd();
    }

    if (egg.shooting || egg.exploding) {
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_ALPHA_TEST);
        glBindTexture(GL_TEXTURE_2D, egg.img);
        glBegin(GL_QUADS);
            glColor4f(1.0, 1.0, 1.0, egg.alpha);
            glTexCoord2i(0, 1);		//Define texture coordinates
            glVertex2f(egg.x - egg.r, egg.y + egg.r);
            glTexCoord2i(0, 0);
            glVertex2f(egg.x - egg.r, egg.y - egg.r);
            glTexCoord2i(1, 0);
            glVertex2f(egg.x + egg.r, egg.y - egg.r);
            glTexCoord2i(1, 1);
            glVertex2f(egg.x + egg.r, egg.y + egg.r);
        glEnd();
    	glBindTexture(GL_TEXTURE_2D, 0);
    	glDisable(GL_ALPHA_TEST);
        /*
        glBegin(GL_LINE_LOOP);
        my_draw_circle(egg.x, egg.y, egg.r);
        glEnd();*/
    }
    glDisable(GL_BLEND);
}



void end_egg() {
}
