#include <allegro.h>
#include <alleggl.h>
#include <math.h>
#include <stdlib.h>

#include "input_map.h"
#include "resources.h"
#include "modes.h"
#include "battle.h"
#include "options.h"



static MODE mode = MODE_OPTIONS;
static DATAFILE *data = NULL;
static int level = 0;
static int unlocked = 0;


int init() {
    set_input_map(KEY_UP, KEY_DOWN, KEY_LEFT, KEY_RIGHT, KEY_A, KEY_S);
    if (mode == MODE_GAME) {
        if (!init_battle(data)) {
            return 0;
        }
        set_battle_level(level);
    }
    if (mode == MODE_OPTIONS) {
        if (!init_options(data, unlocked)) {
            return 0;
        }
    }

    return 1;
}



RESULT update() {
    if (mode == MODE_GAME) {
        return update_battle();
    }
    if (mode == MODE_OPTIONS) {
        level = 0;
        return update_options();
    }
}



void draw() {
    if (mode == MODE_GAME) {
        draw_battle();
    }
    if (mode == MODE_OPTIONS) {
        draw_options();
    }
}



void end() {
    if (mode == MODE_GAME) {
        end_battle();
    }
    if (mode == MODE_OPTIONS) {
        end_options();
    }
}



static int game_timer;



void inc_game_timer()
{
    game_timer++;
}

END_OF_FUNCTION(inc_game_timer);



void init_timer()
{
    LOCK_VARIABLE(game_timer);
    LOCK_FUNCTION(inc_game_timer);
    install_int_ex(inc_game_timer, BPS_TO_TIMER(60));
    game_timer = 0;
}



void end_timer()
{
    remove_int(inc_game_timer);
}



RESULT play_game() {
    RESULT res;
    if (!init()) {
        return RES_ERR;
    }
    while (1) {
        while (game_timer <= 0) {
            rest(1);
        }
        while (game_timer > 0) {
            --game_timer;
            poll_keyboard();
            res = update();
        }
        if (res != RES_CONTINUE) {
            break;
        }
        if (key[KEY_ESC]) {
            res = RES_EXIT;
            break;
        }
        glClear(GL_COLOR_BUFFER_BIT);
        draw();
        glFlush();
        allegro_gl_flip();
    }
    end();
    return res;
}



int main(void)
{
    RESULT res;

    srand(0);
    allegro_init();
    install_allegro_gl();

    install_timer();
    install_mouse();

    data = load_datafile("parody.dat");
    if (!data) {
        allegro_message("Can't load datafile");
        return 0;
    }

    allegro_gl_clear_settings();
	allegro_gl_set(AGL_COLOR_DEPTH, 16);
    allegro_gl_set(AGL_DOUBLEBUFFER, 1);
    allegro_gl_set(AGL_SUGGEST, AGL_DOUBLEBUFFER | AGL_COLOR_DEPTH);

    set_gfx_mode(GFX_OPENGL_WINDOWED, 640, 480, 0, 0);
    install_keyboard();

    glAlphaFunc(GL_GREATER, 0.5);
    allegro_gl_set_texture_format(GL_RGBA);
    glClearColor(0.0, 0.0, 0.0, 0.0);	//Sets the color to clear to
    glColor4f(1.0, 1.0, 1.0, 1.0);
    init_timer();

    load_resources(data);
    // ---
    do {
        res = play_game();

        if (res == RES_ERR) {
            allegro_message("Error");
            break;
        }

        switch (res) {
            case RES_GO_BATTLE:
                mode = MODE_GAME;
                break;
            case RES_WIN:
                ++level;
                if (level >= MAX_LEVEL) {
                    unlocked = 1;
                    mode = MODE_OPTIONS;
                }
                break;
            case RES_LOSE:
                mode = MODE_OPTIONS;
                break;
            default: res = RES_EXIT;
        }
    } while (res != RES_EXIT);

    // ---
    end_timer();
	set_gfx_mode (GFX_TEXT, 0, 0, 0, 0);	//Rest the graphics mode to plain text. This is good practice.

    unload_datafile(data);

    return 0;
}

END_OF_MAIN()
