#include <allegro.h>
#include <alleggl.h>

#include "parody.h"
#include "resources.h"
#include "input_map.h"
#include "options.h"


#define OPTION_FONT_SIZE 90
#define OPTION_X    -0.7
#define OPTION_1_Y  -0.0
#define OPTION_2_Y  -0.2



typedef enum {
    OPTION_START,
    OPTION_EXIT,
} OPTIONS;



GLuint options_bg;
static int k_pressed;
static OPTIONS option;
static FONT *options_font = NULL;


int init_options(DATAFILE *data, int unlocked) {

    k_pressed = 0;

    options_bg = get_resource(unlocked ? OPTIONS_2 : OPTIONS_1);

    allegro_gl_set_texture_format(GL_ALPHA8);
    options_font = allegro_gl_convert_allegro_font((FONT *) data[DAT_FONT_SMALL].dat,
	                                              AGL_FONT_TYPE_TEXTURED, OPTION_FONT_SIZE);

    if (!options_font) {
        return 0;
    }

    return 1;
}



RESULT update_options() {
    INPUT_MAP *map = get_input_map();

    clear_keybuf();

    if (!k_pressed && (key[map->up] || key[map->down])) {
        option = option == OPTION_START ? OPTION_EXIT : OPTION_START;
        k_pressed = 1;
    }
    if (!k_pressed && (key[KEY_ENTER])) {
        return option == OPTION_START ? RES_GO_BATTLE : RES_EXIT;
        k_pressed = 1;
    }
    if (k_pressed) {
        if (!key[map->up] && !key[map->down] && !key[KEY_ENTER]) {
            k_pressed = 0;
        }
    }
    return RES_CONTINUE;
}



void draw_options() {
    int sel = makecol(255,255,255);
    int unsel = makecol(128,128,128);

    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, options_bg);
    glBegin(GL_QUADS);
        glTexCoord2i(0, 1);		//Define texture coordinates
        glVertex2f(-1, 1);
        glTexCoord2i(0, 0);
        glVertex2f(-1, -1);
        glTexCoord2i(1, 0);
        glVertex2f(1, -1);
        glTexCoord2i(1, 1);
        glVertex2f(1, 1);
    glEnd();
	glBindTexture(GL_TEXTURE_2D, 0); // Unbind The Texture

    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    allegro_gl_printf(options_font, OPTION_X, OPTION_1_Y, 0, option == OPTION_START ? sel : unsel, "START");
    allegro_gl_printf(options_font, OPTION_X, OPTION_2_Y, 0, option == OPTION_EXIT ? sel : unsel, "EXIT");
    glDisable(GL_BLEND);
    glColor4f(1.0, 1.0, 1.0, 1.0);

}



void end_options() {
    if (options_font) {
        allegro_gl_destroy_font(options_font);
    }
}
