#include <allegro.h>
#include <alleggl.h>
#include <math.h>

#include "parody.h"
#include "resources.h"
#include "util.h"
#include "collision.h"
#include "stage.h"

#define BLOCKS_HEIGHT   -0.5
#define BLOCK_SPEED     0.005
#define BLOCK_RADIUS    0.2
#define NUM_BLOCKS      5
#define MAX_SPEED_TIME  10



typedef struct BLOCK BLOCK;



struct STAGE {
    GLuint img;
    int speed_timer;
    float blocks_speed;
    float sin_reference;
};



struct BLOCK {
    float x, y;
    float r;
    GLuint img;
};



BLOCK blocks[NUM_BLOCKS] = {
    { -1.0, BLOCKS_HEIGHT, BLOCK_RADIUS, 0, },
    { -0.5, BLOCKS_HEIGHT, BLOCK_RADIUS, 0, },
    {  0.0, BLOCKS_HEIGHT, BLOCK_RADIUS, 0, },
    {  0.5, BLOCKS_HEIGHT, BLOCK_RADIUS, 0, },
    {  1.0, BLOCKS_HEIGHT, BLOCK_RADIUS, 0, },
};



static STAGE stage;



int init_stage(DATAFILE *data) {
    BITMAP *bmp;
    GLuint img;
    int i;

    stage.sin_reference = 0.0;
    stage.blocks_speed = 0.0;
    stage.speed_timer = MAX_SPEED_TIME;

    stage.img = get_resource(DAT_BG);
    img = get_resource(DAT_PLATFORM);

    for (i = 0; i < NUM_BLOCKS; i++) {
        blocks[i].y = BLOCKS_HEIGHT + -0.1 * sin((stage.sin_reference + blocks[i].x) * M_PI);
        blocks[i].img = img;
    }

}



COLLISION_RESULT stage_get_collision(COLLISION_DATA *c) {
    int i;
    COLLISION_DATA block_c;
    COLLISION_RESULT res;
    res.collision = 0;
    for (i = 0; i < NUM_BLOCKS; i++) {
        block_c.x = blocks[i].x;
        block_c.y = blocks[i].y;
        block_c.r = blocks[i].r * 0.8;
        res = check_collision(c, &block_c);
        if (res.collision) {
            break;
        }
    }
    return res;
}



void update_stage() {
    int i;
    int accel_now = 0;

    for (i = 0; i < NUM_BLOCKS; i++) {
        blocks[i].y = BLOCKS_HEIGHT + -0.1 * sin((stage.sin_reference + blocks[i].x) * M_PI);
        blocks[i].x += stage.blocks_speed;
        if (blocks[i].x > 1.0) {
            blocks[i].x -= 2.0;
        }
        if (blocks[i].x < -1.0) {
            blocks[i].x += 2.0;
        }
    }

    stage.sin_reference += 0.01;
    if (stage.sin_reference > 1.0) {
        stage.sin_reference = -1.0;
    }

    if (stage.speed_timer > 0) {
        stage.speed_timer--;
    } else {
        stage.speed_timer = MAX_SPEED_TIME;
        accel_now = 1;
    }

    if (stage.blocks_speed < BLOCK_SPEED && accel_now) {
        stage.blocks_speed += BLOCK_SPEED * 0.01;
    }
}



void draw_stage_bg() {
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, stage.img);
    glBegin(GL_QUADS);
        glTexCoord2i(0, 1);		//Define texture coordinates
        glVertex2f(-1, 1);
        glTexCoord2i(0, 0);
        glVertex2f(-1, -1);
        glTexCoord2i(1, 0);
        glVertex2f(1, -1);
        glTexCoord2i(1, 1);
        glVertex2f(1, 1);
    glEnd();
	glBindTexture(GL_TEXTURE_2D, 0); // Unbind The Texture
}

void draw_stage() {

    int i;

    for (i = 0; i < NUM_BLOCKS; i++) {
        glColor3f(1.0, 1.0, 1.0);
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_ALPHA_TEST);
        glBindTexture(GL_TEXTURE_2D, blocks[i].img);
        glBegin(GL_QUADS);
            glTexCoord2i(0, 1);		//Define texture coordinates
            glVertex2f(blocks[i].x - blocks[i].r, blocks[i].y + blocks[i].r);
            glTexCoord2i(0, 0);
            glVertex2f(blocks[i].x - blocks[i].r, blocks[i].y - blocks[i].r);
            glTexCoord2i(1, 0);
            glVertex2f(blocks[i].x + blocks[i].r, blocks[i].y - blocks[i].r);
            glTexCoord2i(1, 1);
            glVertex2f(blocks[i].x + blocks[i].r, blocks[i].y + blocks[i].r);
        glEnd();
    	glBindTexture(GL_TEXTURE_2D, 0);
    	glDisable(GL_ALPHA_TEST);

        /*
        glBegin(GL_LINE_LOOP);
        glColor3f(1.0, 0.0, 0.0);
        my_draw_circle(blocks[i].x, blocks[i].y, blocks[i].r);
        glEnd(); */
    }
}



void end_stage() {
}
