#include <allegro.h>
#include <alleggl.h>

#include "parody.h"
#include "resources.h"
#include "util.h"
#include "collision.h"
#include "troll.h"

#define MIN_TROLL_RADIUS 0.5
#define TARGET_INCREASE 0.0006
#define MAX_ADVANCE_TIME TROLL_MAX_LEVEL
#define MAX_TROLL_HITS 3
#define TROLL_LINE_X  -0.90
#define TROLL_LINE_Y  -0.90
#define TROLL_LINE_W  1.8
#define TROLL_LINE_H  0.05



struct TROLL {
    int difficulty;
    float x, y;
    float r;
    float distance;
    GLuint img;
    int advance_timer;
    float target_r;
    int hits;
};



static TROLL troll;



int init_troll(DATAFILE *data) {
    BITMAP *bmp;

    troll.x = 0.0;
    troll.y = 0.0;
    troll.r = MIN_TROLL_RADIUS;
    troll.distance = TROLL_TOTAL_DISTANCE;
    troll.advance_timer = MAX_ADVANCE_TIME - troll.difficulty;
    troll.target_r = 0.12;
    troll.hits = 0;

    troll.img = get_resource(DAT_TROLL);

    return 1;
}



void troll_set_difficulty(int d) {
    troll.difficulty = d;
}



int troll_get_difficulty() {
    return troll.difficulty;
}



float troll_get_distance() {
    return troll.distance;
}



void troll_add_hit() {
    if (troll.hits < MAX_TROLL_HITS) {
        ++troll.hits;
    }
}



int troll_is_defeated() {
    return troll.hits == MAX_TROLL_HITS;
}



COLLISION_RESULT troll_get_collision(COLLISION_DATA *c) {
    COLLISION_DATA troll_c;
    COLLISION_RESULT res;
    troll_c.x = troll.x;
    troll_c.y = troll.y;
    troll_c.r = troll.target_r;

    res = check_collision(c, &troll_c);

    if (res.collision) {
        troll.hits++;
    }

    return res;
}



void update_troll() {

    int advance_now = 0;

    troll.r = MIN_TROLL_RADIUS + (TROLL_TOTAL_DISTANCE - troll.distance);

    if (troll.advance_timer > 0) {
        troll.advance_timer--;
    } else {
        troll.advance_timer = MAX_ADVANCE_TIME - troll.difficulty;
        advance_now = 1;
    }

    if (troll.distance > 0.0 && advance_now) {
        troll.distance -= 5 * TARGET_INCREASE;
        troll.target_r += TARGET_INCREASE;
    }
}



void draw_troll() {

    glColor4f(1.0, 1.0, 1.0, 1.0);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_ALPHA_TEST);
    glBindTexture(GL_TEXTURE_2D, troll.img);
    glBegin(GL_QUADS);
        glTexCoord2i(0, 1);		//Define texture coordinates
        glVertex2f(troll.x - troll.r, troll.y + troll.r);
        glTexCoord2i(0, 0);
        glVertex2f(troll.x - troll.r, troll.y - troll.r);
        glTexCoord2i(1, 0);
        glVertex2f(troll.x + troll.r, troll.y - troll.r);
        glTexCoord2i(1, 1);
        glVertex2f(troll.x + troll.r, troll.y + troll.r);
    glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_ALPHA_TEST);

    glColor4f(1.0, 1.0, 1.0, 1.0);
}



void draw_troll_advance() {

    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	// Additive blending
    glEnable(GL_BLEND);

    glBegin(GL_POLYGON);
        glColor4f(0.0, 1.0, 0.0, 1.0);
        glVertex2f(TROLL_LINE_X, TROLL_LINE_Y);
        glVertex2f(TROLL_LINE_X, TROLL_LINE_Y - TROLL_LINE_H);
        glColor4f(1.0, 0.0, 0.0, 1.0);
        glVertex2f(TROLL_LINE_X + TROLL_LINE_W, TROLL_LINE_Y - TROLL_LINE_H);
        glVertex2f(TROLL_LINE_X + TROLL_LINE_W, TROLL_LINE_Y);
        glColor4f(1.0, 1.0, 1.0, 1.0);
    glEnd();

    glBegin(GL_POLYGON);
    glColor4f(1.0, 1.0, 1.0, 0.7);
    my_draw_circle(
                    TROLL_LINE_X + TROLL_TOTAL_DISTANCE - (TROLL_LINE_W / TROLL_TOTAL_DISTANCE * troll.distance),
                    TROLL_LINE_Y - TROLL_LINE_H * 0.5,
                    TROLL_LINE_H
                  );
    glColor4f(1.0, 1.0, 1.0, 1.0);
    glEnd();

    glDisable(GL_BLEND);
}



void end_troll() {
}
